package com.qmth.ops.api.controller.admin;

import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.biz.domain.LlmModel;
import com.qmth.ops.biz.domain.LlmPromptTemplate;
import com.qmth.ops.biz.service.LlmClientService;
import com.qmth.ops.biz.service.LlmModelService;
import com.qmth.ops.biz.service.LlmPromptTemplateService;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/llm/model")
public class LlmModelController {

    @Resource
    private LlmModelService llmModelService;

    @Resource
    private LlmPromptTemplateService llmPromptTemplateService;

    @Resource
    private LlmClientService llmClientService;

    @PostMapping("/list")
    public List<LlmModel> list(@RequestParam Long supplierId) {
        return llmModelService.listBySupplier(supplierId);
    }

    @PostMapping("/insert")
    public LlmModel insert(@RequestAttribute AdminSession adminSession, LlmModel model) {
        adminSession.hasPermission(Permission.LLM_MODEL_INSERT);
        model = llmModelService.insert(model);
        llmClientService.updateByModel(model);
        return model;
    }

    @PostMapping("/update")
    public LlmModel update(@RequestAttribute AdminSession adminSession, LlmModel model) {
        adminSession.hasPermission(Permission.LLM_MODEL_EDIT);
        llmModelService.update(model);
        llmClientService.updateByModel(model);
        return llmModelService.getById(model.getId());
    }

    @PostMapping("/prompt_template/list")
    public List<LlmPromptTemplate> getPromptTemplate(@RequestParam Long modelId, @RequestParam LlmAppType appType) {
        return llmPromptTemplateService.findByModelAndAppType(modelId, appType);
    }

    @PostMapping("/prompt_template/insert")
    public LlmPromptTemplate insertPromptTemplate(@RequestAttribute AdminSession adminSession,
            LlmPromptTemplate template) {
        adminSession.hasPermission(Permission.LLM_MODEL_EDIT);
        llmPromptTemplateService.insert(template);
        return template;
    }

    @PostMapping("/prompt_template/update")
    public LlmPromptTemplate updatePromptTemplate(@RequestAttribute AdminSession adminSession,
            LlmPromptTemplate template) {
        adminSession.hasPermission(Permission.LLM_MODEL_EDIT);
        llmPromptTemplateService.update(template);
        return llmPromptTemplateService.findById(template.getId());
    }

    @PostMapping("/prompt_template/delete")
    public Long deletePromptTemplate(@RequestAttribute AdminSession adminSession, @RequestParam Long id) {
        adminSession.hasPermission(Permission.LLM_MODEL_EDIT);
        llmPromptTemplateService.delete(llmPromptTemplateService.findById(id));
        return id;
    }

}
