package com.qmth.ops.api.controller.admin;

import com.qmth.boot.core.ai.model.ocr.OcrType;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.api.security.Permission;
import com.qmth.ops.api.vo.OcrSupplierVO;
import com.qmth.ops.biz.domain.OcrSupplier;
import com.qmth.ops.biz.service.OcrClientService;
import com.qmth.ops.biz.service.OcrSupplierService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/ocr/supplier")
public class OcrSupplierController {

    @Resource
    private OcrSupplierService ocrSupplierService;

    @Resource
    private OcrClientService ocrClientService;

    @PostMapping("/list")
    public List<OcrSupplierVO> list() {
        return ocrSupplierService.list().stream().map(OcrSupplierVO::new).collect(Collectors.toList());
    }

    @PostMapping("/detail")
    public OcrSupplier detail(@RequestParam Long id) {
        return ocrSupplierService.getById(id);
    }

    @PostMapping("/insert")
    public OcrSupplier insert(@RequestAttribute AdminSession adminSession, OcrSupplier supplier) {
        adminSession.hasPermission(Permission.OCR_SUPPLIER_INSERT);
        ocrSupplierService.insert(supplier);
        ocrClientService.init();
        return ocrSupplierService.getById(supplier.getId());
    }

    @PostMapping("/update")
    public OcrSupplier update(@RequestAttribute AdminSession adminSession, OcrSupplier supplier) {
        adminSession.hasPermission(Permission.OCR_SUPPLIER_EDIT);
        ocrSupplierService.update(supplier);
        ocrClientService.init();
        return ocrSupplierService.getById(supplier.getId());
    }

    @PostMapping("/test")
    public Object test(@RequestParam Long id, @RequestParam OcrType type, @RequestParam MultipartFile image)
            throws Exception {
        Map<String, String> result = new HashMap<>();
        result.put("text", ocrClientService.forImage(id, type, image.getBytes()));
        return result;
    }

}
