package com.qmth.ops.biz.ai.client.azure.llm;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.boot.core.exception.NotFoundException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.ops.biz.ai.client.ChatApiClient;
import com.qmth.ops.biz.ai.client.ChatApiConfig;
import com.qmth.ops.biz.ai.exception.ChatRequestError;
import okhttp3.Headers;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

public class AzureChatClient extends ChatApiClient {

    private static final String AUTH_HEADER_NAME = "api-key";

    private static final String MODEL_PLACEHOLDER = "{model}";

    public AzureChatClient(ChatApiConfig config) {
        super(config);
        config.setUrl(config.getUrl().replace(MODEL_PLACEHOLDER, config.getModel()));
    }

    @Override
    protected Headers buildHeader(Headers.Builder headerBuilder, LlmAppType appType) {
        return headerBuilder.add(AUTH_HEADER_NAME, getConfig().getSecret()).build();
    }

    @Override
    protected Object buildRequest(ChatRequest request, LlmAppType appType) {
        return request;
    }

    @Override
    protected ChatResult buildResult(byte[] data, ObjectMapper mapper) throws IOException {
        return mapper.readValue(data, AzureChatResult.class).buildResult();
    }

    @Override
    protected ChatResult handleError(byte[] data, int statusCode, ObjectMapper mapper) {
        String message = data != null ? new String(data, StandardCharsets.UTF_8) : null;
        switch (statusCode) {
        case 400:
            throw new ChatRequestError(message != null ? message : "chat request error");
        case 404:
            throw new NotFoundException(message != null ? message : "chat resource not found");
        default:
            throw new StatusException(message != null ? message : "chat model error");
        }
    }
}
