package com.qmth.ops.biz.service;

import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.ai.model.llm.LlmAppType;
import com.qmth.ops.biz.ai.client.ChatApiClient;
import com.qmth.ops.biz.ai.client.ChatApiConfig;
import com.qmth.ops.biz.ai.exception.ChatClientNotFound;
import com.qmth.ops.biz.domain.LlmModel;
import com.qmth.ops.biz.domain.LlmSupplier;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class LlmClientService {

    private static final Logger log = LoggerFactory.getLogger(LlmClientService.class);

    @Resource
    private LlmSupplierService supplierService;

    @Resource
    private LlmModelService modelService;

    private Map<Long, ChatApiClient> chatApiClientMap = new HashMap<>();

    public synchronized void init() {
        chatApiClientMap.clear();
        List<LlmSupplier> supplierList = supplierService.list();
        for (LlmSupplier supplier : supplierList) {
            List<LlmModel> modelList = modelService.listBySupplier(supplier.getId());
            for (LlmModel model : modelList) {
                initChatApiClient(supplier, model);
            }
        }
    }

    public synchronized void updateBySupplier(LlmSupplier supplier) {
        List<LlmModel> modelList = modelService.listBySupplier(supplier.getId());
        for (LlmModel model : modelList) {
            initChatApiClient(supplier, model);
        }
    }

    public synchronized void updateByModel(LlmModel model) {
        LlmSupplier supplier = supplierService.getById(model.getSupplierId());
        initChatApiClient(supplier, model);
    }

    private void initChatApiClient(LlmSupplier supplier, LlmModel model) {
        try {
            String className = ChatApiClient.class.getName().replace("ChatApiClient", supplier.getChatClientClass());
            ChatApiConfig config = new ChatApiConfig(supplier, model);
            Class<?> clientClass = Class.forName(className);
            chatApiClientMap.put(model.getId(),
                    (ChatApiClient) clientClass.getConstructor(ChatApiConfig.class).newInstance(config));
        } catch (Exception e) {
            log.error("Chat api client init error, supplier={}, class={}, model={}", supplier.getName(),
                    supplier.getChatClientClass(), model.getName());
        }
    }

    public ChatResult chat(ChatRequest request, Long modelId, LlmAppType appType) throws Exception {
        ChatApiClient client = chatApiClientMap.get(modelId);
        if (client == null) {
            throw new ChatClientNotFound(modelId);
        }
        return client.call(request, appType);
    }
}
